﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Problems.PFSP
{ /// <summary>
  /// An operator which evaluates PFSP solutions given in path representation using the rounded Euclidean distance metric.
  /// </summary>
    [Item("CmaxMakeSpanEvaluator", "An operator which evaluates PFSP solutions given in matrix representation using Cmax")]
    [StorableClass]
    public sealed class CmaxMakeSpanEvaluator : MakeSpanEvaluator
    {
        [StorableConstructor]
        private CmaxMakeSpanEvaluator(bool deserializing) : base(deserializing) { }
        private CmaxMakeSpanEvaluator(CmaxMakeSpanEvaluator original, Cloner cloner) : base(original, cloner) { }
        public CmaxMakeSpanEvaluator() : base() { }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new CmaxMakeSpanEvaluator(this, cloner);
        }

        protected override double CalculateMakespan(DoubleMatrix m, Permutation p)
        {
            DoubleMatrix calculatedTime = new DoubleMatrix(m.Rows, m.Columns);
            double runtimeCur;
            double runtimePrev = 0;

            double runtimePrevMachine;
            double runtimePrevJobOnThisMachine;

            int[] permutation = p.ToArray();

            for (var machineIdx = 0; machineIdx < m.Rows; machineIdx++)
            {
                for (var jobIdx = 0; jobIdx < m.Columns; jobIdx++)
                {
                    runtimeCur = m[machineIdx, permutation[jobIdx]];
                    runtimePrev = 0;

                    //Simply add to calculatedTimeMatrix
                    if (jobIdx == 0 && machineIdx == 0)
                    {
                        //Take default (runtimePrev = 0)
                    }
                    //Add the time + the previous time
                    else if (machineIdx == 0)
                    {
                        runtimePrev = calculatedTime[machineIdx, jobIdx - 1];
                    }
                    //Check the job-time from the previous machine and previos job and take the greater
                    else if (jobIdx == 0)
                    {
                        runtimePrev = calculatedTime[machineIdx - 1, jobIdx];
                    }
                    //X X X X X
                    //X O O O O
                    //X O O O O
                    //X O O O O
                    else
                    {
                        runtimePrevMachine = calculatedTime[machineIdx - 1, jobIdx];
                        runtimePrevJobOnThisMachine = calculatedTime[machineIdx, jobIdx - 1];
                        runtimePrev = runtimePrevMachine > runtimePrevJobOnThisMachine ? runtimePrevMachine : runtimePrevJobOnThisMachine;
                    }
                    calculatedTime[machineIdx, jobIdx] = runtimeCur + runtimePrev;
                }
            }
            //return the makespan(the total runtime from start till the last job is finished)
            return calculatedTime[calculatedTime.Rows - 1, calculatedTime.Columns - 1];
        }
    }
}
